module Handler.Quotes where

import Import
import Data.Time (getCurrentTime, formatTime)
import System.Locale (defaultTimeLocale)
import Settings.StaticFiles
import qualified Data.HashMap.Strict as H
import Data.Aeson (ToJSON, toJSON)
import Data.Aeson.Types (Pair)


quoteWidget :: QuoteId -> Quote -> Widget
quoteWidget quoteId quote = do
    $(widgetFile "quote")


getQuotesR :: Handler RepHtmlJson
getQuotesR = do
    quotes <- runDB $ selectList [] [Desc QuoteAdded]
    json <- mapM (\(Entity qid q) -> quoteJson (qid, q)) quotes
    let html = do
            setTitle "Quotes"
            $(widgetFile "quotes")
    defaultLayoutJson html json


jsonWithExtras :: (ToJSON a) => [Pair] -> a -> Value
jsonWithExtras extras entity = case toJSON entity of
    Object o -> Object $ foldl (\m (k, v) -> H.insert k v m) o extras
    _ -> error "Unexpected JSON representation of entity"


jsonWithId :: (ToJSON a, ToJSON b) => a -> b -> Value
jsonWithId eid = jsonWithExtras ["id" .= eid]


quoteJson :: (QuoteId, Quote) -> Handler Value
quoteJson (qid, quote) = do
    r <- getUrlRender
    return $ jsonWithExtras
        ["id" .= qid,
         "voteUp" .= r (QuoteUpR qid),
         "voteDown" .= r (QuoteDownR qid),
         "self" .= r (QuoteR qid),
         "rating" .= quoteRating quote,
         "votes" .= quoteVotes quote] quote


getQuoteR :: QuoteId -> Handler RepHtmlJson
getQuoteR qid = do
    quote <- runDB $ get404 qid
    json <- quoteJson (qid, quote)
    let html = do
            setTitle "Quote"
            quoteWidget qid quote
    defaultLayoutJson html json


quoteForm :: Form Quote
quoteForm = renderDivs $ Quote
    <$> areq textField "Quote" Nothing
    <*> aformM (liftIO getCurrentTime)
    <*> pure 0
    <*> pure 0


getNewQuoteR :: Handler RepHtml
getNewQuoteR = do
    ((_, newQuoteWidget), enctype) <- generateFormPost quoteForm
    defaultLayout $ do
        setTitle "New quote"
        $(widgetFile "newquote")


postNewQuoteR :: Handler RepHtml
postNewQuoteR = do
    ((result, newQuoteWidget), enctype) <- runFormPost quoteForm
    case result of
        FormSuccess quote -> do
            quoteId <- runDB $ insert quote
            redirect $ QuoteR quoteId
        _ -> defaultLayout $ do
            setTitle "New quote"
            $(widgetFile "newquote")


postQuoteUpR :: QuoteId -> Handler RepHtml
postQuoteUpR quoteId = do
    runDB $ update quoteId [QuoteVotesFor +=. 1]
    redirect (QuoteR quoteId)


postQuoteDownR :: QuoteId -> Handler RepHtml
postQuoteDownR quoteId = do
    runDB $ update quoteId [QuoteVotesAgainst +=. 1]
    redirect (QuoteR quoteId)
