$(document).ready -> 
    window.FlyingCircus = {}

    class FlyingCircus.Quote extends Backbone.Model
        voteUp: ->
            d = $.post(@get('voteUp'), undefined, undefined, 'json')
            d.done (data) =>
                @voted = true
                @set data
        voteDown: ->
            d = $.post(@get('voteDown'), undefined, undefined, 'json')
            d.done (data) =>
                @voted = true
                @set data


    class FlyingCircus.QuoteList extends Backbone.Collection
        model: FlyingCircus.Quote

    FlyingCircus.quoteList = new FlyingCircus.QuoteList

    class FlyingCircus.QuoteView extends Backbone.View
        className: 'quote-container'
        tagName: 'div'
        template: _.template $('#quote-template').html()
        initialize: ->
            @model.bind('change', @render)
            @model.bind('remove', @remove)
        render: =>
            @$el.html @template(@model.toJSON())
            @$el.find('a').attr('href', @model.get 'self')
            @$el.on 'click', 'a', (event) ->
                FlyingCircus.router.navigate(@pathname, trigger: true)
                event.preventDefault()
            if @model.voted
                @disableVoting(true)
                @hideVoting()
            return @
        remove: =>
            @$el.remove()
        events:
            'click button.vote-up':   'voteUp'
            'click button.vote-down': 'voteDown'
        voteUp: ->
            @disableVoting(true)
            d = @model.voteUp()
            d.fail (f) =>
                @flash()
                @disableVoting(false)
        voteDown: ->
            @disableVoting(true)
            d = @model.voteDown()
            d.fail (f) =>
                @flash()
                @disableVoting(false)
        flash: (cssClass='error-flash') ->
            el = $('<div />').addClass(cssClass)
            @$el.find('.quote').append(el)
            el.fadeIn(200).fadeOut(600).queue(() ->
                el.remove()
                el.dequeue())
        disableVoting: (disabled=true) ->
            @$el.find('.controls button').prop('disabled', disabled)
        hideVoting: ->
            controls = @$el.find('.controls')
            buttons = controls.find('button')
            controls.css opacity: 1
            buttons.fadeOut 200
            controls.find('.voted').fadeIn 200

    class FlyingCircus.AppView extends Backbone.View
        el: $('#app')
        initialize: ->
            ql = FlyingCircus.quoteList
            ql.bind('add',   @addOne)
            ql.bind('reset', @reset)
            $('.nav-block > ul').on 'click', 'a', (event) ->
                FlyingCircus.router.navigate(@pathname, trigger: true)
                event.preventDefault()
        addOne: (quote) =>
            view = new FlyingCircus.QuoteView(model: quote)
            @$el.append view.render().el
        reset: =>
            @$el.empty()
            FlyingCircus.quoteList.each (quote) => @addOne quote

    class FlyingCircus.CircusRouter extends Backbone.Router
        routes:
            '':            'overview'
            'quotes':      'quotes'
            'quotes/:qid': 'quote'
        overview: ->
            FlyingCircus.quoteList.reset []
        quotes: ->
            $.getJSON('/quotes').success (data) ->
                FlyingCircus.quoteList.reset data
        quote: (qid) ->
            $.getJSON('/quotes/' + qid).success (data) ->
                FlyingCircus.quoteList.reset [data]

    FlyingCircus.router = new FlyingCircus.CircusRouter
    Backbone.history.start pushState: true

    FlyingCircus.app = new FlyingCircus.AppView
